-- A DB can't have any users connected so you have to clear everyone out before you can restore.
--Sometimes this can be really difficult if there are a lot of users doing a lot of work. Or
--if you have an app that keeps reconnecting.
--One of the best ways to get around the reconnection problem is to identify which logins keep reconnecting
--and disable them before you kick them out of the DB.  That way they can't reconnect.
BACKUP DATABASE CDC
TO DISK = 'c:\MyBackups\CDCFull.Original'
WITH init, format

USE MASTER
GO
ALTER DATABASE [CDC] SET  SINGLE_USER WITH ROLLBACK IMMEDIATE
GO
DROP DATABASE CDC
GO
RESTORE DATABASE CDC
FROM DISK = 'C:\MyBackups\CDCFull.Original'

--OK, that was an easy restore where the DB didn't exist anymore.  And while that's common enough, I'll
--go ahead and say it's not the most common type of restore you'll ever do.
--Restoring with the DB in tact is probably the most common restore you'll ever do.
--But that's not to say that you can't drop the existing one first every time.

USE MASTER
GO
ALTER DATABASE [CDC] SET  SINGLE_USER WITH ROLLBACK IMMEDIATE
GO

RESTORE HEADERONLY--ONLY
FROM DISK = 'C:\MyBackups\CDCFull.Original'

RESTORE DATABASE CDC2
FROM DISK = 'C:\MyBackups\CDCFull.Original'
WITH REPLACE,
MOVE 'CDC' TO 'C:\MyDBs\CDC2.mdf',
MOVE 'CDC_log' TO 'C:\MyDBs\CDC_log2.ldf'



------Basic log restore.
------------------------
------------------------


-------------------SETUP Scenario----------------------
BACKUP DATABASE [CDC]
TO DISK = 'C:\MyBackups\CDCFull.LogRestoreBase'
WITH init

UPDATE dbo.[MyCDC1]
SET col1 = col1 + '--1st log backup.'
WHERE ID <=25 

--SELECT TOP 30 * FROM dbo.[MyCDC1]

BACKUP LOG [CDC]
TO DISK = 'C:\MyBackups\CDCLog.Log1'
WITH init

UPDATE dbo.[MyCDC1]
SET col1 = col1 + '--2nd log backup.'
WHERE ID BETWEEN 26 AND 35 

BACKUP LOG [CDC]
TO DISK = 'C:\MyBackups\CDCLog.Log2'
WITH init

UPDATE dbo.[MyCDC1]
SET col1 = col1 + '--3rd log backup.'
WHERE ID BETWEEN 36 AND 50 

BACKUP LOG [CDC]
TO DISK = 'C:\MyBackups\CDCLog.Log3'
WITH init

-----------------Perform Restore-----------------------
USE [master]
GO

ALTER DATABASE [CDC] SET  SINGLE_USER WITH ROLLBACK IMMEDIATE
GO
ALTER DATABASE [CDC]
SET multi_user

RESTORE DATABASE [CDC]
FROM DISK = 'C:\MyBackups\CDCFull.LogRestoreBase'
WITH REPLACE, norecovery

RESTORE LOG [CDC]
FROM DISK = 'C:\MyBackups\CDCLog.Log1'
WITH norecovery

RESTORE LOG [CDC]
FROM DISK = 'C:\MyBackups\CDCLog.Log2'
WITH norecovery

RESTORE LOG [CDC]
FROM DISK = 'C:\MyBackups\CDCLog.Log3'
WITH recovery

RESTORE DATABASE [CDC]
WITH standby = 'c:\mystandby.txt'


------Notes on norecovery/standby-----------------
/*
there's a difference between using norecovery and standby.  
With norecovery the DB is left in an inconsistent state and unavailable, even for querying.
This is because you might be in the middle of a transaction and SQL needs to next log to complete it so
it keeps the DB closed completely.

Standby takes a filename as a parameter (standby = 'c:\standby.txt')and it uses this file to hold rollback
transactions.  This leaves the DB in a readonly state, but still able to recover more logs.

With recovery rolls back any incomplete transactions and opens the DB to users.

Here's an example of how this might work.
You have a long running transaction.  Let's say it takes 2hrs.  Your first log backup happens before it
starts, and the 2nd starts in the middle of it, and the 3rd happens once it completes.  
If you restore the DB to the 2nd log backup and recover the DB, then you'll have half a trans.
And w/o the rest of the trans the DB can't guarantee consistency.  So it rolls back the first half of that
trans when you use with recovery.  Recovery also leaves you unable to restore any further logs.
Think about it.  If you were able to restore that last log after recovering the DB, you'd have the
2nd half of that transaction and you'd still be inconsistent.  There could be other trans that come after
that which could rely on the data in the first half of that orig trans.  So since the entire trans isn't there, 
sql doesn't allow you to restore any more logs.

The problem comes in where you may need to look at the data to decide where to stop.  Say you wanted to do a
STOPAT but you don't know where to stop.  So what sql does when you use standby is it rolls back the
transactions that are incomplete, but stores them in the standby file(the official name).  This way if
you decide to recover, all it has to do is open the DB, but if you decide to restore another log, 
that new log restore replays the transactions in that standby file first, and then completes its own restore.
And this process keeps going until your DB is in a consistent state and has been recovered.
*/

------Basic Differential restore.
------------------------
------------------------
USE [CDC]
GO

BACKUP DATABASE [CDC]
TO DISK = 'C:\MyBackups\CDCFull.DiffRestoreBase'
WITH INIT


UPDATE dbo.[MyCDC1]
SET col1 = col1 + '--1st diff backup.'
WHERE ID <=25 

--SELECT TOP 30 * FROM dbo.[MyCDC1]

BACKUP database [CDC]
TO DISK = 'C:\MyBackups\CDCDiff.Diff1'
WITH Differential, INIT

UPDATE dbo.[MyCDC1]
SET col1 = col1 + '--2nd diff backup.'
WHERE ID BETWEEN 26 AND 35 

BACKUP database [CDC]
TO DISK = 'C:\MyBackups\CDCDiff.Diff2'
WITH Differential, INIT

UPDATE dbo.[MyCDC1]
SET col1 = col1 + '--3rd diff backup.'
WHERE ID BETWEEN 36 AND 50 

BACKUP database [CDC]
TO DISK = 'C:\MyBackups\CDCDiff.Diff3'
WITH Differential, INIT


--**********
--Now go look at the files.  Can anyone tell me why they're the same size?
--Didn't we already establish that they'll grow each time you take one?


--------------Perform Diff Restore-----------------------
USE [master]
GO

RESTORE DATABASE [CDC]
FROM DISK = 'C:\MyBackups\CDCFull.DiffRestoreBase'
WITH REPLACE, norecovery 

RESTORE DATABASE [CDC]
FROM DISK = 'C:\MyBackups\CDCDiff.Diff3'
WITH  norecovery

RESTORE DATABASE [CDC]
WITH recovery


RESTORE DATABASE [CDC]
FROM DISK = 'C:\MyBackups\cdcfull.original'
WITH REPLACE